!Higher order differentiation up to order 9 for the Faddeeva function 
!using Faa di Bruno's formula  (other Taylor coeffcients are set to 0).
!
!based on the f77 subroutine
!      ALGORITHM 680, COLLECTED ALGORITHMS FROM ACM.
!      THIS WORK PUBLISHED IN TRANSACTIONS ON MATHEMATICAL SOFTWARE,
!      VOL. 16, NO. 1, PP. 47.
!      http://www.netlib.org/toms-2014-06-10/680
!
!Author: I. Charpentier

SUBROUTINE Faa_Faddeeva (z,v)
!Use Taylor coefficiets defined in HOAD_types
USE HOAD_types
TYPE(Complex16d)::z    !complex Taylor series argument 
TYPE(Complex16d)::v    !complex Taylor series result v=w(z) 
!Local variables
INTEGER::k             !Loop index
INTEGER::fac(0:Ck)     !Factorial "k!"
COMPLEX*16::phi(0:Ck)  !compound function
REAL*8::phir,phii      !real and imaginary parts of phii
LOGICAL::flag          !flag of wofz

fac(0)=1;
DO k = 1,Ck
!factorial computation for scaling  
  fac(k)=fac(k-1)*k;z%d(k) = z%d(k)*fac(k); 
ENDDO

!Formula 7.1.20
 CALL wofz(REAL(z%d(0)),AIMAG(z%d(0)),phir,phii,flag)
 phi(0)=phir+(0.d0,1.d0)*phii
 phi(1)=-2*z%d(0)*phi(0)+2*(0.d0,1.d0)/sqrt(3.14159265358979323846d0)
 do k=0,6
   phi(k+2)=-2*z%d(0)*phi(k+1)-2*(k+1)*phi(k)
 enddo

!Bell polynomials (GENERAL STATEMENTS FOR FAA DI BRUNO FORMULA)
!order 0
v%d(0)=    phi(0)
!order 1
v%d(1)=    phi(1)*z%d(1)
! order 2
v%d(2)=    phi(2)*z%d(1)**2&
 &      +  phi(1)*z%d(2)
!order 3
v%d(3)=    phi(3)*z%d(1)**3       &
 &      +  phi(2)*3.*z%d(2)*z%d(1)&
 &      +  phi(1)*z%d(3)
!order 4
v%d(4)=    phi(4)*z%d(1)**4       &
 &      +6*phi(3)*z%d(2)*z%d(1)**2&
 &      +4*phi(2)*z%d(3)*z%d(1)   &
 &      +3*phi(2)*z%d(2)**2       &
 &      +  phi(1)*z%d(4)
!order 5
v%d(5) =   phi(5) *z%d(1)**5        &
 &    +10 *phi(4) *z%d(2) *z%d(1)**3&
 &    +10 *phi(3) *z%d(3) *z%d(1)**2&
 &    +15 *phi(3) *z%d(2)**2 *z%d(1)&
 &    + 5 *phi(2) *z%d(4) *z%d(1)   &
 &    +10 *phi(2) *z%d(3) *z%d(2)   &
 &    +    phi(1) *z%d(5)
!order 6
v%d(6) =   phi(6) *z%d(1)**6             &
 &    +15 *phi(5) *z%d(2) *z%d(1)**4     &
 &    +20 *phi(4) *z%d(3) *z%d(1)**3     &
 &    +45 *phi(4) *z%d(2)**2 *z%d(1)**2  &
 &    +15 *phi(3) *z%d(4) *z%d(1)**2     &
 &    +60 *phi(3) *z%d(3) *z%d(2) *z%d(1)&
 &    +15 *phi(3) *z%d(2)**3             &
 &    +10 *phi(2) *z%d(3)**2             &
 &    +15 *phi(2) *z%d(4) *z%d(2)        &
 &    + 6 *phi(2) *z%d(5) *z%d(1)        & 
 &    +    phi(1) *z%d(6)
!order 7
v%d(7) =  phi(7)  *z%d(1)**7                &
 &    +21 *phi(6) *z%d(2) *z%d(1)**5        &
 &    +35 *phi(5) *z%d(3) *z%d(1)**4        &
 &   +105 *phi(5) *z%d(2)**2 *z%d(1)**3     &
 &    +35 *phi(4) *z%d(4) *z%d(1)**3        &
 &   +210 *phi(4) *z%d(3) *z%d(2) *z%d(1)**2& 
 &   +105 *phi(4) *z%d(2)**3 *z%d(1)        &
 &    +21 *phi(3) *z%d(5) *z%d(1)**2        &
 &   +105 *phi(3) *z%d(4) *z%d(2) *z%d(1)   &   
 &    +70 *phi(3) *z%d(3)**2 *z%d(1)        &
 &   +105 *phi(3) *z%d(3) *z%d(2)**2        &
 &    + 7 *phi(2) *z%d(6) *z%d(1)           &
 &    +21 *phi(2) *z%d(5) *z%d(2)           &
 &    +35 *phi(2) *z%d(4) *z%d(3)           &
 &    +    phi(1) *z%d(7)
!order 8
v%d(8) =     phi(8)    *z%d(1)**8                &
 &         +28 *phi(7) *z%d(2) *z%d(1)**6        &
 &         +56 *phi(6) *z%d(3) *z%d(1)**5        &
 &        +210 *phi(6) *z%d(2)**2 *z%d(1)**4     &
 &         +70 *phi(5) *z%d(4) *z%d(1)**4        & 
 &        +560 *phi(5) *z%d(3) *z%d(2) *z%d(1)**3&    
 &        +420 *phi(5) *z%d(2)**3 *z%d(1)**2     &
 &         +56 *phi(4) *z%d(5) *z%d(1)**3        &
 &        +420 *phi(4) *z%d(4) *z%d(2) *z%d(1)**2&
 &        +280 *phi(4) *z%d(3)**2 *z%d(1)**2     &   
 &        +840 *phi(4) *z%d(3) *z%d(2)**2 *z%d(1)&   
 &        +105 *phi(4) *z%d(2)**4                &
 &         +28 *phi(3) *z%d(6) *z%d(1)**2        &
 &        +168 *phi(3) *z%d(5) *z%d(2) *z%d(1)   &
 &        +280 *phi(3) *z%d(4) *z%d(3) *z%d(1)   &
 &        +210 *phi(3) *z%d(4) *z%d(2)**2        & 
 &        +280 *phi(3) *z%d(3)**2 *z%d(2)        &
 &         + 8 *phi(2) *z%d(7) *z%d(1)           &
 &         +28 *phi(2) *z%d(6) *z%d(2)           &
 &         +56 *phi(2) *z%d(5) *z%d(3)           &
 &         +35 *phi(2) *z%d(4)**2                &
 &         +    phi(1) *z%d(8)
!order 9 and higher arbitrarily set to 0
v%d(9:Ck)=0.d0
!unscaling
 DO i = 0,Ck
  v%d(i) = v%d(i)/fac(i); 
  z%d(i) = z%d(i)/fac(i); ENDDO
END SUBROUTINE Faa_Faddeeva

!Copyright (c) 2016, CNRS and University of Strasbourg, France
!All rights reserved.

!Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

!1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

!2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

!3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

!THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
