!Higher order automatic differentiation up to order Ck for the aimag function (imaginary part of a complex number)
!where Ck is a global variable defining the current truncation order of differentiation
!
!The HOAD_aimag module allows to package the different versions of aimag so that they can be imported into  program units to be automatically differentiated at higher order.
!
!aimag is overloaded for a Taylor series implemented as a Complex16 variable (see HOAD_types)
!
!Author: I. Charpentier

MODULE HOAD_aimag
  USE HOAD_types
  IMPLICIT NONE
!Convention: 
!  aimag for aimag
!  type Complex16d
  INTERFACE AIMAG
!Overloaded aimag: the unique argument is a Complex16d Taylor series
    MODULE PROCEDURE aimag_complex16d
  END INTERFACE AIMAG
CONTAINS 
!-----
!OVERLOADING r=aimag(a): the unique argument is a Complex16d Taylor series
!                      : the result is a Real8d Taylor series
  FUNCTION aimag_complex16d(a) result(r)
    TYPE(Complex16d),INTENT(IN)::a
    TYPE(Real8d)::r
    INTEGER :: j
!Recurrence formula: r_k=aimag(a_k), for all k
    DO j = 0,Ck 
      r%d(j) = aimag(a%d(j))
    ENDDO
  END FUNCTION aimag_complex16d
END MODULE HOAD_aimag

!Copyright (c) 2016, CNRS and University of Strasbourg, France
!All rights reserved.

!Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

!1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

!2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

!3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

!THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
