!Higher order automatic differentiation up to order Ck for the uminus operator (a unique operand)
!where Ck is a global variable defining the current truncation order of differentiation
!
!The HOAD_sub module allows to package the different versions of - so that they can be imported into  program units to be automatically differentiated at higher order.
!
!- is overloaded for Taylor series. Related HOAD types are defined in HOAD_types. 
!  * Real8d for double precision/real*8 Taylor coefficients 
!  * Complex16 for complex*16 Taylor coefficients 
!
!Author: I. Charpentier

MODULE HOAD_uminus
  USE HOAD_types
  IMPLICIT NONE
!Combination of types for the overloaded uminus -
!Convention: 
!  uminus for -, 
!  one operand of type Real8d, Complex16d
  INTERFACE OPERATOR(-)
    MODULE PROCEDURE uminus_Real8d
    MODULE PROCEDURE uminus_Complex16d
  END INTERFACE OPERATOR(-)
CONTAINS 

!-----
!OVERLOADING r=-a: THE OPERAND IS A TAYLOR SERIES (Real8d or Complex16d)
  FUNCTION uminus_Real8d(a) RESULT(r)
    TYPE(Real8d),INTENT(IN)::a
    TYPE(Real8d)::r
!Recurrence formula: r_k=-a_k, for all k
    r%d(0:Ck)=-a%d(0:Ck)
  END FUNCTION uminus_Real8d

  FUNCTION uminus_Complex16d(a) RESULT(r)
    TYPE(Complex16d),INTENT(IN)::a
    TYPE(Complex16d)::r
!Recurrence formula: r_k=-a_k, for all k
    r%d(0:Ck)=-a%d(0:Ck)
  END FUNCTION uminus_Complex16d
END MODULE HOAD_uminus

!Copyright (c) 2016, CNRS and University of Strasbourg, France
!All rights reserved.

!Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

!1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

!2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

!3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

!THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
