#Commands for figure 1a
#Although the Taylor coefficients of the refractive index n (real part of the complex refractive index ntilde) and the extinction coefficient kappa (imaginary part of ntilde) are sometimes negative, they are her plotted in a special "log-log scale".
#
#The proposed "log-log scale" consists in the plot
# * positive Taylor coefficients in a classical log-log scale 
# * negative Taylor coefficients in a mirrored log-log scale (plot positive value top down with a change of the ticks )
#
#This is possible thanks to the chosen storage: wavelength, absolute value, sign, ...
#that enable, through awk commands, to select values to be plotted using one or the other log-logfrom their

## uncomment to plot figures in eps files
set terminal postscript eps enhanced color font 'Helvetica,24'


#####
##Figure 1a
set title 'Refractive index: HOAD based on (13)'
set output 'Ag-nlog.eps'   #for postscript eps terminal
set key top right 
set multiplot layout 2,1  

## top plot: classical log-log scale
set logscale x; set logscale y
##set margins to ensure that top and bottom plots are same size
set bmargin 0; set tmargin 2.5; set lmargin 6
##set range
set yrange [1.e-8:1e10]
set xrange [0.2e-6:12.e-6]
##define ticks
unset xtics           #no xticks for the top plot
set ytics (1,1e5, 1e10) out  nomirror
set ytics add ("~0" 1e-8); set ytics add ("1E-5" 1e-5); set ytics add ("1E5" 1e5)
set ytics add ("1E10" 1e10)
##plot values with positive sign
plot "< awk '$3>0 { print $1, $2 }' n.dat" using 1:2 w p pt 18 ps 0.49 title ' n_0',"< awk '$5>0 { print $1, $4 }' n.dat" using 1:2 w p pt 18 ps 0.49 title 'n_1', "< awk '$7>0 { print $1, $6 }' n.dat" using 1:2 w p pt 18 ps 0.49 title 'n_2', "< awk '$9>0 { print $1, $8 }' n.dat" using 1:2 w p pt 18 ps 0.49 title 'n_3', "< awk '$11>0 { print $1, $10 }' n.dat" using 1:2 w p pt 18 ps 0.49 title 'n_4', "< awk '$13>0 { print $1, $12 }' n.dat" using 1:2 w p pt 18 ps 0.49 title 'n_5', "< awk '$15>0 { print $1, $14 }' n.dat" using 1:2 w p pt 18 ps 0.49 title 'n_6'

## bottom plot: mirrored log-log scale
unset title; unset key
set logscale x; set logscale y
## set margins to match first plot
set bmargin 2.5; set tmargin 0
##set range
set yrange [1.e-8:1e10] reverse
set xrange [0.2e-6:12.e-6]
##define ticks
set ytics (1,1e5, 1e10) out nomirror
set ytics add ("-1" 1); set ytics add ("-1E-5" 1e-5); set ytics add ("-1E5"  1e5)
set ytics add ("-1E10" 1e10)
set xtics out nomirror offset 0,0.4  # this offset compresses the bottom whitespace
set xtics add ("" .2e-6,"" .3e-6,"" .4e-6,"" .5e-6,"" .6e-6,"" .7e-6,"" .8e-6,"" .9e-6,"1" 1.e-6)
set xtics add ("10" 1.e-5,"" 1.1e-5,"" 1.2e-5)
## xlabel 
set xlabel 'Wavelength {/Symbol l}_0 ({/Symbol m}m)' offset 0,0.8
##plot values with a negative sign
plot "< awk '$3<0.5 { print $1, $2 }' n.dat" using 1:2 title '',"< awk '$5<0.5 { print $1, $2 }' n.dat" using 1:2  w p pt 18 ps 0.49 title ' ',"< awk '$7<1 { print $1, $4 }' n.dat" using 1:2 w p pt 18 ps 0.49 title ' ', "< awk '$9<1 { print $1, $8 }' n.dat" using 1:2 w p pt 18 ps 0.49 title ' ', "< awk '$11<1 { print $1, $10 }' n.dat" using 1:2 w p pt 18 ps 0.49 title 'n_4', "< awk '$13<1 { print $1, $12 }' n.dat" using 1:2 w p pt 18 ps 0.49 title 'n_5', "< awk '$15<1 { print $1, $14 }' n.dat" using 1:2 w p pt 18 ps 0.49 title 'n_6'

unset multiplot
reset
#pause -1

#####
##Figure 1c
set title 'Extinction coefficient: HOAD based on (13)'
set output 'Ag-klog.eps'   #for postscript eps terminal
set key top right 
set multiplot layout 2,1  

## top plot: classical log-log scale
set logscale x; set logscale y
#set margins to ensure that top and bottom plots are same size
set bmargin 0; set tmargin 2.5; set lmargin 6
##set range
set yrange [1.e-8:1e10]
set xrange [0.2e-6:12.e-6]
##define ticks
unset xtics           #no xticks for the top plot
set ytics (1,1e5, 1e10) out  nomirror
set ytics add ("~0" 1e-8); set ytics add ("1E-5" 1e-5); set ytics add ("1E5" 1e5)
set ytics add ("1E10" 1e10)
##plot values with positive sign
plot "< awk '$3>0 { print $1, $2 }' k.dat" using 1:2 w p pt 18 ps 0.49 title ' {/Symbol k}_0',"< awk '$5>0 { print $1, $4 }' k.dat" using 1:2 w p pt 18 ps 0.49 title '{/Symbol k}_1', "< awk '$7>0 { print $1, $6 }' k.dat" using 1:2 w p pt 18 ps 0.49 title '{/Symbol k}_2', "< awk '$9>0 { print $1, $8 }' k.dat" using 1:2 w p pt 18 ps 0.49 title '{/Symbol k}_3', "< awk '$11>0 { print $1, $10 }' k.dat" using 1:2 w p pt 18 ps 0.49 title '{/Symbol k}_4', "< awk '$13>0 { print $1, $12 }' k.dat" using 1:2 w p pt 18 ps 0.49 title '{/Symbol k}_5', "< awk '$15>0 { print $1, $14 }' k.dat" using 1:2 w p pt 18 ps 0.49 title '{/Symbol k}_6'

### second (bottom) plot
## bottom plot: mirrored log-log scale
unset title; unset key
set logscale x; set logscale y
## set margins to match first plot
set bmargin 2.5; set tmargin 0
##set range
set yrange [1.e-8:1e10] reverse
set xrange [0.2e-6:12.e-6]
##define ticks
set ytics (1,1e5, 1e10) out nomirror
set ytics add ("-1" 1); set ytics add ("-1E-5" 1e-5); set ytics add ("-1E5"  1e5)
set ytics add ("-1E10" 1e10)
set xtics out nomirror offset 0,0.4  # this offset compresses the bottom whitespace
set xtics add ("" .2e-6,"" .3e-6,"" .4e-6,"" .5e-6,"" .6e-6,"" .7e-6,"" .8e-6,"" .9e-6,"1" 1.e-6)
set xtics add ("10" 1.e-5,"" 1.1e-5,"" 1.2e-5)
## xlabel 
set xlabel 'Wavelength {/Symbol l}_0 ({/Symbol m}m)' offset 0,0.8
##plot values with a negative sign
plot "< awk '$3<0.5 { print $1, $2 }' k.dat" using 1:2 title '',"< awk '$5<0.5 { print $1, $2 }' k.dat" using 1:2  w p pt 18 ps 0.49 title ' ',"< awk '$7<1 { print $1, $4 }' k.dat" using 1:2 w p pt 18 ps 0.49 title ' ', "< awk '$9<1 { print $1, $8 }' k.dat" using 1:2 w p pt 18 ps 0.49 title ' ', "< awk '$11<1 { print $1, $10 }' k.dat" using 1:2 w p pt 18 ps 0.49 title 'n4', "< awk '$13<1 { print $1, $12 }' k.dat" using 1:2 w p pt 18 ps 0.49 title 'n5', "< awk '$15<1 { print $1, $14 }' k.dat" using 1:2 w p pt 18 ps 0.49 title 'n6'

unset multiplot

reset
#pause -1


#Copyright (c) 2016, CNRS and University of Strasbourg, France
#All rights reserved.

#Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

#1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

#2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

#3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.



