!Program devoted to the calculation of Taylor coefficients of the complex refractive index 
!of silver at optical wavelengths (Brendel-Bormann)
!
!Author: I. Charpentier

PROGRAM main_HOAD_ntilde
!HOAD operator overloading library
USE HOAD_types; USE HOAD_add;  USE HOAD_div;    USE HOAD_prod; 
USE HOAD_power; USE HOAD_sub;  USE HOAD_uminus; USE HOAD_sqrt;
USE HOAD_real;  USE HOAD_aimag; USE HOAD_exp; USE HOAD_faddeeva;

IMPLICIT NONE
!Taylor series
TYPE(Real8d)    ::lambda           !wavelength
TYPE(Complex16d)::epsilon,n        !dielectric function, complex refractive index
!Material
 CHARACTER*3    ::material 
!Local variables 
REAL*8::lambdamin,lambdamax,dlambda !lower and upper bounds for lambda, increment
INTEGER::nstep                      !increment number
INTEGER::j,k                        !loop indices for the wavelength and differentiation order
!Signs for plot purposes
INTEGER signreal(0:32), signimag(0:32)

!-----
!Initialization
!Maximum order of truncation
 Ckmax=32
 Ck=Ckmax
!Wavelength definitions
lambdamin=0.1d-6                    !lower bound
lambdamax=12.6d-6                   !upper bound
nstep=5000                          !increment number
dlambda=(lambdamax-lambdamin)/nstep !increment
!Taylor series related to the wavelength 
lambda%d(:)=0.d0; 
lambda%d(0)=lambdamin; !value
lambda%d(1)=1.d-6;     !perturbation
!Material choice for the Brendel-Bormann model
material='Ag'  !Silver is the unique one for now 

!-----
!Files to store the real and imaginary parts of first seven Taylor coefficients
!absolute values and signs are stored so as to plot them along a "negative log scale" (where needed)
OPEN(2, FILE='n.dat'); OPEN(3, FILE='k.dat'); 
REWIND(2); REWIND(3)

!-----
!Iterations on the wavelength to compute Taylor coefficients needed to Figures 1a and 1c
DO j=1,nstep
!Call the Brendel-Bormann model
  CALL BBmodel(lambda,material,epsilon,n)
!Compute signs (for plot purpose)
  signreal(:)=0; signimag(:)=0
  DO k=0,Ck
     IF (REAL(n%d(k)).GE.0) signreal(k)=1
     IF (AIMAG(n%d(k)).GE.0) signimag(k)=1
  ENDDO
!Store real parts as absolute values and signs to be plotted in "log-log scale" using the command
!gnuplot loglog.gnu
  WRITE(2,*) lambda%d(0),ABS(REAL(n%d(0))), signreal(0), &
                         ABS(REAL(n%d(1))), signreal(1), &
                         ABS(REAL(n%d(2))), signreal(2), &
  	 		 ABS(REAL(n%d(3))), signreal(3), &
			 ABS(REAL(n%d(4))), signreal(4), &
			 ABS(REAL(n%d(5))), signreal(5), &
			 ABS(REAL(n%d(6))), signreal(6)
!Store imag parts as absolute values and signs to be plotted in "log-log scale" using the command
!gnuplot loglog.gnu
  WRITE(3,*) lambda%d(0),ABS(AIMAG(n%d(0))), signimag(0), &
                         ABS(AIMAG(n%d(1))), signimag(1), &
                         ABS(AIMAG(n%d(2))), signimag(2), &
			 ABS(AIMAG(n%d(3))), signimag(3), &
			 ABS(AIMAG(n%d(4))), signimag(4), &
			 ABS(AIMAG(n%d(5))), signimag(5), &
			 ABS(AIMAG(n%d(6))), signimag(6)
!next increment for lambda (only lambda_0 is impacted since dlambda is a real*8 variable)
  lambda=lambda+dlambda
ENDDO
 CLOSE(2); CLOSE(3)
END PROGRAM main_HOAD_ntilde

!Copyright (c) 2016, CNRS and University of Strasbourg, France
!All rights reserved.

!Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

!1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

!2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

!3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

!THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
