      subroutine othpl ( kf, n, x, pl, dpl )

c*********************************************************************72
c
cc OTHPL computes orthogonal polynomials Tn(x), Un(x), Ln(x) or Hn(x).
c
c  Discussion:
c
c    This procedure computes orthogonal polynomials: Tn(x) or Un(x),
c    or Ln(x) or Hn(x), and their derivatives.
c
c  Licensing:
c
c    This routine is copyrighted by Shanjie Zhang and Jianming Jin.  However, 
c    they give permission to incorporate this routine into a user program 
c    provided that the copyright is acknowledged.
c
c  Modified:
c
c    08 July 2012
c
c  Author:
c
c    Shanjie Zhang, Jianming Jin
c
c  Reference:
c
c    Shanjie Zhang, Jianming Jin,
c    Computation of Special Functions,
c    Wiley, 1996,
c    ISBN: 0-471-11963-6,
c    LC: QA351.C45.
c
c  Parameters:
c
c    Input, integer KT, the function code:
c    1 for Chebyshev polynomial Tn(x)
c    2 for Chebyshev polynomial Un(x)
c    3 for Laguerre polynomial Ln(x)
c    4 for Hermite polynomial Hn(x)
c
c    Input, integer N, the order.
c
c    Input, double precision X, the argument.
c
c    Output, double precision PL(0:N), DPL(0:N), the value and derivative of
c    the polynomials of order 0 through N at X.
c
      implicit none

      integer n

      double precision a
      double precision b
      double precision c
      double precision dpl(0:n)
      double precision dy0
      double precision dy1
      double precision dyn
      integer k
      integer kf
      double precision pl(0:n)
      double precision x
      double precision y0
      double precision y1
      double precision yn

      a = 2.0D+00
      b = 0.0D+00
      c = 1.0D+00
      y0 = 1.0D+00
      y1 = 2.0D+00 * x
      dy0 = 0.0D+00
      dy1 = 2.0D+00
      pl(0) = 1.0D+00
      pl(1) = 2.0D+00 * x
      dpl(0) = 0.0D+00
      dpl(1) = 2.0D+00

      if ( kf .eq. 1 ) then
        y1 = x
        dy1 = 1.0D+00
        pl(1) = x
        dpl(1) = 1.0D+00
      else if ( kf .eq. 3 ) then
        y1 = 1.0D+00 - x
        dy1 = -1.0D+00
        pl(1) = 1.0D+00 - x
        dpl(1) = -1.0D+00
      end if

      do k = 2, n

        if ( kf .eq. 3 ) then
          a = -1.0D+00 / k
          b = 2.0D+00 + a
          c = 1.0D+00 + a
        else if ( kf .eq. 4 ) then
          c = 2.0D+00 * ( k - 1.0D+00 )
        end if

        yn = ( a * x + b ) * y1 - c * y0
        dyn = a * y1 + ( a * x + b ) * dy1 - c * dy0
        pl(k) = yn
        dpl(k) = dyn
        y0 = y1
        y1 = yn
        dy0 = dy1
        dy1 = dyn

      end do
 
      return
      end
