!Subroutine devoted to the test of the HOAD of the 2F1 hypergeometric function 
!Comparisons are made with the Faa di Bruno'formula results and Identity 15.1.8 [Abraham&Stegun]

!Author: I. Charpentier

SUBROUTINE test_HOAD_2F1
USE HOAD_types; USE HOAD_add;  USE HOAD_div;    USE HOAD_prod; 
USE HOAD_power; USE HOAD_sub;  USE HOAD_uminus; USE HOAD_sqrt;
USE HOAD_real;  USE HOAD_aimag; USE HOAD_exp; USE HOAD_2F1; 
IMPLICIT NONE
REAL*8          ::a,b,c  !parameters
TYPE(Complex16d)::z      !evaluation point
TYPE(Complex16d)::v,vFaa,v_identity !Taylor coefficients computed with the ODE formula, Faa di Bruno formula and the identity 15.1.8 of Abraham&Stegun
INTEGER         ::k      !loop index over the orders

 Ckmax=32; Ck=Ckmax
!Choice of the parameters
a=(1.3d0,0.d0); b=(1.2d0,0.d0); c=(1.2d0,0.d0);
!Choice of the evaluation point
z%d(:)=(0.d0,0.d0); z%d(0)=(.4d0,.8d0);z%d(1)=(1.d0,0.d0)
!Taylor coefficients for 2F1(a,b,c,z,v), HOAD following the ODE
 CALL fun_2F1(a,b,c,z,v)  
!Taylor coefficients for 2F1(a,b,c,z,w), HOAD following Faa di Bruno
 CALL FAa_hygfz(a,b,c,z,vFaa)!Faa di Bruno's formula
!Taylor coefficients of the identity 15.1.8 of Abraham&Stegun
v_identity=(1-z)**(-a)

!Results presented in latex form
WRITE(*,*)'\begin{table}'
WRITE(*,*)'\caption{Real and imaginary parts of the Taylor coefficients of $_2F_1(a,b;b;z)$ and'
WRITE(*,*)' $(1-z)^{-a}$ for $a=1.3$, $b=1.2$ and $z_0=(.4d0,.8d0)$ and $z_k=(1.d0,0.d0)$ for $k=1,..,32$. Table 7 from $[7]$.}'
WRITE(*,*)'\begin{tabular}{llll}\hline'
WRITE(*,*)'            &  Real parts               &                           &\\\hline'
WRITE(*,*)'          k &  Faa di Bruno             &  ODE formula              & $(1-z)**(-a)$\\\hline'
DO k=0,5
       WRITE(*,*)k,'&',real(vFaa%d(k)),'&',real(v%d(k)),'&',    real(v_identity%d(k)),'\\'; 
ENDDO
k=8 ;  WRITE(*,*)k,'&',real(vFaa%d(k)),'&',real(v%d(k)),'&',real(v_identity%d(k)),'\\'
k=16;  WRITE(*,*)k,'&',real(vFaa%d(k)),'&',real(v%d(k)),'&',real(v_identity%d(k)),'\\'
k=24;  WRITE(*,*)k,'&',real(vFaa%d(k)),'&',real(v%d(k)),'&',real(v_identity%d(k)),'\\'
k=32;  WRITE(*,*)k,'&',real(vFaa%d(k)),'&',real(v%d(k)),'&',real(v_identity%d(k)),'\\\hline'
WRITE(*,*)'            &  Imaginary parts          &                           &\\\hline'
DO k=0,5
       WRITE(*,*)k,'&',imag(vFaa%d(k)),'&',imag(v%d(k)),'&',imag(v_identity%d(k)),'\\'; 
ENDDO
k=8 ;  WRITE(*,*)k,'&',imag(vFaa%d(k)),'&',imag(v%d(k)),'&',imag(v_identity%d(k)),'\\'
k=16;  WRITE(*,*)k,'&',imag(vFaa%d(k)),'&',imag(v%d(k)),'&',imag(v_identity%d(k)),'\\'
k=24;  WRITE(*,*)k,'&',imag(vFaa%d(k)),'&',imag(v%d(k)),'&',imag(v_identity%d(k)),'\\'
k=32;  WRITE(*,*)k,'&',imag(vFaa%d(k)),'&',imag(v%d(k)),'&',imag(v_identity%d(k)),'\\\hline'
WRITE(*,*)'\end{tabular}'
WRITE(*,*)'\end{table}'
END SUBROUTINE test_HOAD_2F1

!Copyright (c) 2016, CNRS and University of Strasbourg, France
!All rights reserved.

!Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

!1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

!2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

!3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

!THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
