!Subroutine devoted to the test of the HOAD of the Bessel function Jn
!Comparisons are made with the Faa di Bruno'formula results and Identity 9.1.27 [Abraham&Stegun]

!Author: I. Charpentier

SUBROUTINE test_HOAD_BesselJn
USE HOAD_types; USE HOAD_add;  USE HOAD_div;    USE HOAD_prod; 
USE HOAD_power; USE HOAD_sub;  USE HOAD_uminus; USE HOAD_sqrt;
USE HOAD_real;  USE HOAD_aimag; USE HOAD_exp; USE HOAD_BesselJn;

IMPLICIT NONE
INTEGER::n  !parameter
TYPE(Real8d)::x,v4,v5,v6,vid,vF5,Faa_dbesJn
INTEGER::k  !loop index over the orders

 Ck=32; 
!Choice of the parameter
n=5; 
!Choice of the evaluation point
x%d(:)=1.d0
!Taylor coefficients for J_4(x), J5_(x) and J_6(x), HOAD following the ODE
v4=DBESJN(4,x); v5=DBESJN(5,x); v6=DBESJN(6,x);
vid=10*v5/x
!Taylor coefficients for J5_(x), HOAD following Faa di Bruno
vF5=Faa_dbesJn(5,x); 

!Results presented in latex form
WRITE(*,*)'\begin{table}'
WRITE(*,*)'\caption{Taylor coefficients of the Bessel function $J_n$ of order $n=5$ '
WRITE(*,*)' and the identity 9.1.27 for $x$ such that $x_k=1$ for $k=0,..,32$. Table 5 from $[2]$.}'
WRITE(*,*)'\begin{tabular}{llll}\hline'
WRITE(*,*) '          k &   $J_5(x)$                &   $J_5(x)$                &$J_{4}(x)+J_{6}(x)=\frac{10}{x}J_{5}(x)$ \\'
WRITE(*,*) '            & (Fa\`a di Bruno)          & (ODE)                     &  (ODE)\\\hline'
DO k=0,8
       WRITE(*,*)k,'&',   vF5%d(k),'&', v5%d(k),'&',v4%d(k)+v6%d(k)-vid%d(k),'\\'
ENDDO
k=16;  WRITE(*,*)k,'&',   vF5%d(k),'&', v5%d(k),'&',v4%d(k)+v6%d(k)-vid%d(k),'\\'
k=24;  WRITE(*,*)k,'&',   vF5%d(k),'&', v5%d(k),'&',v4%d(k)+v6%d(k)-vid%d(k),'\\'
k=32;  WRITE(*,*)k,'&',   vF5%d(k),'&', v5%d(k),'&',v4%d(k)+v6%d(k)-vid%d(k),'\\\hline'
WRITE(*,*)'\end{tabular}'
WRITE(*,*)'\end{table}'
110 FORMAT(I2,' & ',e21.16,' & ',e21.16,' & ',e21.16,'\\')
END SUBROUTINE test_HOAD_BesselJn

!Copyright (c) 2016, CNRS and University of Strasbourg, France
!All rights reserved.

!Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

!1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

!2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

!3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

!THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
