!Subroutine devoted to the test of the HOAD of the Chebyshev function Tn
!Comparisons are made with the Faa di Bruno'formula results and Identity 22.3.15 [Abraham&Stegun]

!Author: I. Charpentier

SUBROUTINE test_HOAD_ChebyshevTn
USE HOAD_types; USE HOAD_add;  USE HOAD_div;    USE HOAD_prod; 
USE HOAD_power; USE HOAD_sub;  USE HOAD_uminus; USE HOAD_sqrt;
USE HOAD_real;  USE HOAD_aimag; USE HOAD_exp; USE HOAD_cos; 
USE HOAD_ChebyshevTn

IMPLICIT NONE
INTEGER::N  !parameter
TYPE(Real8d)::x,v,cosnx
INTEGER::k  !loop index over the orders

 Ckmax=32; Ck=32; 
!Choice of the parameter
n=10; 
!Choice of the evaluation point
x%d(0)=7.d0/13.d0; x%d(1:Ck)=1.d0;
!Validation considering formula 22.3.15 Abraham&Steun
!Taylor coefficients for T_n(COS(x)), HOAD following the ODE
 CALL ChebyshevTn(n,COS(x),v)
!Taylor coefficients cos(nx)
cosnx=COS(n*x)

!Results presented in latex form
WRITE(*,*)'\begin{table}'
WRITE(*,*)'\caption{Taylor coefficients of $T_n\circ\cos(\theta)$ and $\cos(n\theta)$ '
WRITE(*,*)'for $n=10$, $\theta_0=7/13$ and $\theta_k=1$ for k=1,..,32. Table 6 from $[2]$.}'
WRITE(*,*)'\begin{tabular}{lll}\hline'
WRITE(*,*) '          k &   $T_n(\cos(\theta))$           &  $\cos(n\theta)$                \\'
WRITE(*,*) '            &   (ODE)                   &  (HOAD on Identity)                        \\\hline'
DO k=0,8
       WRITE(*,*)k,'&',   v%d(k),'&', cosnx%d(k),'\\'
ENDDO
k=8 ;  WRITE(*,*)k,'&',   v%d(k),'&', cosnx%d(k),'\\'
k=16;  WRITE(*,*)k,'&',   v%d(k),'&', cosnx%d(k),'\\'
k=24;  WRITE(*,*)k,'&',   v%d(k),'&', cosnx%d(k),'\\'
k=32;  WRITE(*,*)k,'&',   v%d(k),'&', cosnx%d(k),'\\\hline'
WRITE(*,*)'\end{tabular}'
WRITE(*,*)'\end{table}'
110 FORMAT(I2,' & ',e21.16,' & ',e21.16,'\\')
END SUBROUTINE test_HOAD_ChebyshevTn

!Copyright (c) 2016, CNRS and University of Strasbourg, France
!All rights reserved.

!Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

!1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

!2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

!3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

!THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
